@@ -17,7 +17,7 @@ from taskgraph.util.templates import merge
 here = Path(__file__).parent
 
 
-def fake_loader(kind, path, config, parameters, loaded_tasks):
+def fake_loader(kind, path, config, parameters, loaded_tasks, write_artifacts):
     for i in range(3):
         dependencies = {}
         if i >= 1:
@@ -30,7 +30,7 @@ def fake_loader(kind, path, config, parameters, loaded_tasks):
             "attributes": {"_tasknum": str(i)},
             "task": {
                 "i": i,
-                "metadata": {"name": f"t-{i}"},
+                "metadata": {"name": f"{kind}-t-{i}"},
                 "deadline": "soon",
             },
             "dependencies": dependencies,
@@ -62,8 +62,13 @@ class WithFakeKind(TaskGraphGenerator):
             yield FakeKind(kind_name, "/fake", config, graph_config)
 
 
+class FakeGraphConfig(GraphConfig):
+    def register(self):
+        pass
+
+
 def fake_load_graph_config(root_dir):
-    graph_config = GraphConfig(
+    graph_config = FakeGraphConfig(
         {
             "trust-domain": "test-domain",
             "taskgraph": {
@@ -103,7 +108,6 @@ def fake_load_graph_config(root_dir):
         },
         root_dir,
     )
-    graph_config.__dict__["register"] = lambda: None
     return graph_config
 
 
