@@ -22,12 +22,12 @@ def test_get_primary_dependency(make_transform_config):
         "foo": make_task("foo", kind="kind1"),
         "bar": make_task("bar", kind="kind2"),
     }
-    config = make_transform_config(kind_dependencies_tasks=dep_tasks)
     task = {
         "attributes": {},
         "dependencies": {"kind1": "foo", "kind2": "bar"},
     }
+    config = make_transform_config(kind_dependencies_tasks=dep_tasks)
     assert get_primary_dependency(config, task) is None
 
-    task["attributes"]["primary-kind-dependency"] = "kind2"
+    task["attributes"]["primary-dependency-label"] = dep_tasks["bar"].label
     assert get_primary_dependency(config, task) == list(dep_tasks.values())[1]
